@@ -1,13 +1,12 @@
 package XML::Struct;
 # ABSTRACT: Represent XML as data structure preserving element order
-our $VERSION = '0.21'; # VERSION
+our $VERSION = '0.22'; # VERSION
 
 use strict;
 use XML::LibXML::Reader;
-use List::Util qw(first);
-
 use XML::Struct::Reader;
 use XML::Struct::Writer;
+use XML::Struct::Simple;
 
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(readXML writeXML simpleXML removeXMLAttr textValues);
@@ -18,7 +17,7 @@ sub readXML { # ( [$from], %options )
     my %reader_options = (
         map { $_ => delete $options{$_} }
         grep { exists $options{$_} }
-        qw(attributes whitespace path stream simple micro root ns depth)
+        qw(attributes whitespace path stream simple micro root ns depth content)
     );
     if (%options) {
         if (exists $options{from} and keys %options == 1) {
@@ -38,86 +37,10 @@ sub writeXML {
 
 sub simpleXML {
     my ($element, %options) = @_;
-
-    my $attributes = (!defined $options{attributes} or $options{attributes});
-
-    if ($attributes eq 'remove') {
-        $element = removeXMLAttr($element);
-        $attributes = 0;
-    }
-
-    if (defined $options{depth} and $options{depth} !~ /^\+?\d+/) {
-        $options{depth} = undef;
-    }
-
-    if ($options{root}) {
-        my $root = $options{root};
-        $root = $element->[0] if $root =~ /^[+-]?[0-9]+$/;
-
-        $options{depth}-- if defined $options{depth};
-
-        my $hash = $attributes
-                ? _simple(1, [ dummy => {}, [$element] ], $options{depth})
-                : _simple(0, [ dummy => [$element] ], $options{depth});
-
-        return { $root => values %$hash };
-    }
-
-    my $hash = _simple($attributes, $element, $options{depth});
-    $hash = { } unless ref $hash;
-
-    return $hash;
+    XML::Struct::Simple->new(%options)->transform($element);
 }
 
-sub _push_hash {
-    my ($hash, $key, $value, $force) = @_;
-
-    if ( exists $hash->{$key} ) {
-        if ((ref $hash->{$key} || '') ne 'ARRAY') {
-            $hash->{$key} = [ $hash->{$key} ];
-        }
-        push @{$hash->{$key}}, $value;
-    } elsif ( $force ) {
-        $hash->{$key} = [ $value ];
-    } else {
-        $hash->{$key} = $value;
-    }
-}
-
-# hashifies attributes and child elements
-sub _simple {
-    my $with_attributes = shift;
-    my ($children, $attributes) = $with_attributes ? ($_[0]->[2], $_[0]->[1]) : ($_[0]->[1]);
-    my $depth = defined $_[1] ? $_[1] - 1 : undef;
-
-    # empty element or characters only 
-    if (!($attributes and %$attributes) and !first { ref $_ } @$children) {
-        my $text = join "", @$children;
-        return $text ne "" ? $text : { };
-    }
-
-    my $hash = { $attributes ? %$attributes : () };
-
-    foreach my $child ( @$children ) {
-        next unless ref $child; # skip mixed content text
-        if (defined $depth and $depth < 0) {
-            _push_hash( $hash, $child->[0], $child, 1 );
-        } else {
-            _push_hash( $hash, $child->[0] => _simple($with_attributes, $child, $depth) );
-        }
-    }
-
-    return $hash; 
-}
-
-sub removeXMLAttr {
-    my $node = shift;
-    ref $node
-        ? ( $node->[2]
-            ? [ $node->[0], [ map { removeXMLAttr($_) } @{$node->[2]} ] ]
-            : [ $node->[0] ] ) # empty element
-        : $node;               # text node
-}
+*removeXMLAttr = *XML::Struct::Simple::removeXMLAttr;
 
 # TODO: document (better name?)
 sub textValues {
@@ -147,7 +70,7 @@ XML::Struct - Represent XML as data structure preserving element order
 
 =head1 VERSION
 
-version 0.21
+version 0.22
 
 =head1 SYNOPSIS
 
@@ -201,6 +124,10 @@ Parse XML as stream into XML data structures.
 Write XML data structures to XML streams for serializing, SAX processing, or
 creating a DOM object.
 
+=item L<XML::Struct::Simple>
+
+Transform XML data structure into simple form. 
+
 =item L<XML::Struct::Writer::Stream>
 
 Simplified SAX handler for XML serialization.
@@ -224,44 +151,7 @@ Write an XML document/element with L<XML::Struct::Writer>.
 =head2 simpleXML( $element [, %options ] )
 
 Transform an XML document/element into simple key-value format as known from
-L<XML::Simple>: Attributes and child elements are treated as hash keys with
-their content as value. Text elements without attributes are converted to text
-and empty elements without attributes are converted to empty hashes. The
-following options are supported:
-
-=over 4
-
-=item root
-
-Keep the root element (just as option C<KeepRoot> in L<XML::Simple>). In
-addition one can set the name of the root element if a non-numeric value is
-passed.
-
-=item depth
-
-Only transform to a given depth. See L<XML::Struct::Reader> for documentation.
-
-All elements below the given depth are returned unmodified (not cloned) as
-array elements:
-
-    $data = simpleXML($xml, depth => 2)
-    $content = $data->{x}->{y}; # array or scalar (if existing)
-
-=item attributes
-
-Assume input without attributes if set to a true value. The special value
-C<remove> will first remove attributes, so the following three are equivalent:
-
-    my @children = (['a'],['b']);
-
-    simpleXML( [ $name => \@children ], attributes => 0 );
-    simpleXML( removeXMLAttr( [ $name => \%attributes, \@children ] ), attributes => 0 );
-    simpleXML( [ $name => \%attributes, \@children ], attributes => 'remove' );
-
-=back
-
-Key attributes (C<KeyAttr> in L<XML::Simple>) and the option C<ForceArray> are
-not supported yet.
+L<XML::Simple>. See L<XML::Struct::Simple> for configuration options.
 
 =head2 removeXMLAttr( $element )
 
